@@ -21,20 +21,21 @@ import logging
 import os
 import sys
 
+from cliff import columns as cliff_columns
+from glanceclient.common import utils as gc_utils
+from osc_lib.api import utils as api_utils
+from osc_lib.cli import format_columns
+from osc_lib.cli import parseractions
+from osc_lib.command import command
+from osc_lib import utils
+
+from openstackclient.i18n import _
+
 if os.name == "nt":
     import msvcrt
 else:
     msvcrt = None
 
-from glanceclient.common import utils as gc_utils
-from osc_lib.cli import parseractions
-from osc_lib.command import command
-from osc_lib import utils
-import six
-
-from openstackclient.api import utils as api_utils
-from openstackclient.i18n import _
-
 
 CONTAINER_CHOICES = ["ami", "ari", "aki", "bare", "docker", "ova", "ovf"]
 DEFAULT_CONTAINER_FORMAT = 'bare'
@@ -46,19 +47,18 @@ DISK_CHOICES = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2", "vhdx",
 LOG = logging.getLogger(__name__)
 
 
-def _format_visibility(data):
-    """Return a formatted visibility string
+class VisibilityColumn(cliff_columns.FormattableColumn):
+    def human_readable(self):
+        """Return a formatted visibility string
 
-    :param data:
-        The server's visibility (is_public) status value: True, False
-    :rtype:
-        A string formatted to public/private
-    """
+        :rtype:
+            A string formatted to public/private
+        """
 
-    if data:
-        return 'public'
-    else:
-        return 'private'
+        if self._value:
+            return 'public'
+        else:
+            return 'private'
 
 
 class CreateImage(command.ShowOne):
@@ -182,29 +182,16 @@ class CreateImage(command.ShowOne):
             help=_("Set a property on this image "
                    "(repeat option to set multiple properties)"),
         )
-        # NOTE(dtroyer): --owner is deprecated in Jan 2016 in an early
-        #                2.x release.  Do not remove before Jan 2017
-        #                and a 3.x release.
-        project_group = parser.add_mutually_exclusive_group()
-        project_group.add_argument(
+        parser.add_argument(
             "--project",
             metavar="<project>",
             help=_("Set an alternate project on this image (name or ID)"),
         )
-        project_group.add_argument(
-            "--owner",
-            metavar="<project>",
-            help=argparse.SUPPRESS,
-        )
         return parser
 
     def take_action(self, parsed_args):
         image_client = self.app.client_manager.image
 
-        if getattr(parsed_args, 'owner', None) is not None:
-            LOG.warning(_('The --owner option is deprecated, '
-                          'please use --project instead.'))
-
         # Build an attribute dict from the parsed args, only include
         # attributes that were actually set on the command line
         kwargs = {}
@@ -281,8 +268,9 @@ class CreateImage(command.ShowOne):
                     kwargs['data'].close()
 
             info.update(image._info)
-            info['properties'] = utils.format_dict(info.get('properties', {}))
-        return zip(*sorted(six.iteritems(info)))
+            info['properties'] = format_columns.DictColumn(
+                info.get('properties', {}))
+        return zip(*sorted(info.items()))
 
 
 class DeleteImage(command.Command):
@@ -442,8 +430,8 @@ class ListImage(command.Lister):
                 s,
                 columns,
                 formatters={
-                    'is_public': _format_visibility,
-                    'properties': utils.format_dict,
+                    'is_public': VisibilityColumn,
+                    'properties': format_columns.DictColumn,
                 },
             ) for s in data)
         )
@@ -599,29 +587,16 @@ class SetImage(command.Command):
             metavar="<checksum>",
             help=_("Image hash used for verification"),
         )
-        # NOTE(dtroyer): --owner is deprecated in Jan 2016 in an early
-        #                2.x release.  Do not remove before Jan 2017
-        #                and a 3.x release.
-        project_group = parser.add_mutually_exclusive_group()
-        project_group.add_argument(
+        parser.add_argument(
             "--project",
             metavar="<project>",
             help=_("Set an alternate project on this image (name or ID)"),
         )
-        project_group.add_argument(
-            "--owner",
-            metavar="<project>",
-            help=argparse.SUPPRESS,
-        )
         return parser
 
     def take_action(self, parsed_args):
         image_client = self.app.client_manager.image
 
-        if getattr(parsed_args, 'owner', None) is not None:
-            LOG.warning(_('The --owner option is deprecated, '
-                          'please use --project instead.'))
-
         kwargs = {}
         copy_attrs = ('name', 'owner', 'min_disk', 'min_ram', 'properties',
                       'container_format', 'disk_format', 'size', 'store',
@@ -740,5 +715,6 @@ class ShowImage(command.ShowOne):
         if parsed_args.human_readable:
             if 'size' in info:
                 info['size'] = utils.format_size(info['size'])
-        info['properties'] = utils.format_dict(info.get('properties', {}))
-        return zip(*sorted(six.iteritems(info)))
+        info['properties'] = format_columns.DictColumn(
+            info.get('properties', {}))
+        return zip(*sorted(info.items()))
