@@ -14,12 +14,12 @@
 #
 
 import copy
+from unittest import mock
 
 from glanceclient.common import utils as glanceclient_utils
 from glanceclient.v2 import schemas
-import mock
+from osc_lib.cli import format_columns
 from osc_lib import exceptions
-from osc_lib import utils as common_utils
 import warlock
 
 from openstackclient.image.v2 import image
@@ -116,7 +116,7 @@ class TestImageCreate(TestImage):
         self.assertEqual(
             image_fakes.FakeImage.get_image_columns(self.new_image),
             columns)
-        self.assertEqual(
+        self.assertItemEqual(
             image_fakes.FakeImage.get_image_data(self.new_image),
             data)
 
@@ -184,44 +184,10 @@ class TestImageCreate(TestImage):
         self.assertEqual(
             image_fakes.FakeImage.get_image_columns(self.new_image),
             columns)
-        self.assertEqual(
+        self.assertItemEqual(
             image_fakes.FakeImage.get_image_data(self.new_image),
             data)
 
-    def test_image_create_with_unexist_owner(self):
-        self.project_mock.get.side_effect = exceptions.NotFound(None)
-        self.project_mock.find.side_effect = exceptions.NotFound(None)
-
-        arglist = [
-            '--container-format', 'ovf',
-            '--disk-format', 'ami',
-            '--min-disk', '10',
-            '--min-ram', '4',
-            '--owner', 'unexist_owner',
-            '--protected',
-            '--private',
-            image_fakes.image_name,
-        ]
-        verifylist = [
-            ('container_format', 'ovf'),
-            ('disk_format', 'ami'),
-            ('min_disk', 10),
-            ('min_ram', 4),
-            ('owner', 'unexist_owner'),
-            ('protected', True),
-            ('unprotected', False),
-            ('public', False),
-            ('private', True),
-            ('name', image_fakes.image_name),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        self.assertRaises(
-            exceptions.CommandError,
-            self.cmd.take_action,
-            parsed_args,
-        )
-
     def test_image_create_with_unexist_project(self):
         self.project_mock.get.side_effect = exceptions.NotFound(None)
         self.project_mock.find.side_effect = exceptions.NotFound(None)
@@ -318,7 +284,7 @@ class TestImageCreate(TestImage):
         self.assertEqual(
             image_fakes.FakeImage.get_image_columns(self.new_image),
             columns)
-        self.assertEqual(
+        self.assertItemEqual(
             image_fakes.FakeImage.get_image_data(self.new_image),
             data)
 
@@ -527,6 +493,7 @@ class TestImageList(TestImage):
         verifylist = [
             ('public', False),
             ('private', False),
+            ('community', False),
             ('shared', False),
             ('long', False),
         ]
@@ -541,7 +508,7 @@ class TestImageList(TestImage):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.datalist, tuple(data))
+        self.assertListItemEqual(self.datalist, tuple(data))
 
     def test_image_list_public_option(self):
         arglist = [
@@ -550,6 +517,7 @@ class TestImageList(TestImage):
         verifylist = [
             ('public', True),
             ('private', False),
+            ('community', False),
             ('shared', False),
             ('long', False),
         ]
@@ -565,7 +533,7 @@ class TestImageList(TestImage):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.datalist, tuple(data))
+        self.assertListItemEqual(self.datalist, tuple(data))
 
     def test_image_list_private_option(self):
         arglist = [
@@ -574,6 +542,7 @@ class TestImageList(TestImage):
         verifylist = [
             ('public', False),
             ('private', True),
+            ('community', False),
             ('shared', False),
             ('long', False),
         ]
@@ -588,6 +557,31 @@ class TestImageList(TestImage):
             marker=self._image.id,
         )
 
+        self.assertEqual(self.columns, columns)
+        self.assertListItemEqual(self.datalist, tuple(data))
+
+    def test_image_list_community_option(self):
+        arglist = [
+            '--community',
+        ]
+        verifylist = [
+            ('public', False),
+            ('private', False),
+            ('community', True),
+            ('shared', False),
+            ('long', False),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.image_list.assert_called_with(
+            community=True,
+            marker=self._image.id,
+        )
+
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, tuple(data))
 
@@ -598,6 +592,7 @@ class TestImageList(TestImage):
         verifylist = [
             ('public', False),
             ('private', False),
+            ('community', False),
             ('shared', True),
             ('long', False),
         ]
@@ -612,9 +607,52 @@ class TestImageList(TestImage):
             marker=self._image.id,
         )
 
+        self.assertEqual(self.columns, columns)
+        self.assertListItemEqual(self.datalist, tuple(data))
+
+    def test_image_list_shared_member_status_option(self):
+        arglist = [
+            '--shared',
+            '--member-status', 'all'
+        ]
+        verifylist = [
+            ('public', False),
+            ('private', False),
+            ('community', False),
+            ('shared', True),
+            ('long', False),
+            ('member_status', 'all')
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        # In base command class Lister in cliff, abstract method take_action()
+        # returns a tuple containing the column names and an iterable
+        # containing the data to be listed.
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.image_list.assert_called_with(
+            shared=True,
+            member_status='all',
+            marker=self._image.id,
+        )
+
         self.assertEqual(self.columns, columns)
         self.assertEqual(self.datalist, tuple(data))
 
+    def test_image_list_shared_member_status_lower(self):
+        arglist = [
+            '--shared',
+            '--member-status', 'ALl'
+        ]
+        verifylist = [
+            ('public', False),
+            ('private', False),
+            ('community', False),
+            ('shared', True),
+            ('long', False),
+            ('member_status', 'all')
+        ]
+        self.check_parser(self.cmd, arglist, verifylist)
+
     def test_image_list_long_option(self):
         arglist = [
             '--long',
@@ -658,11 +696,11 @@ class TestImageList(TestImage):
             self._image.visibility,
             self._image.protected,
             self._image.owner,
-            common_utils.format_list(self._image.tags),
+            format_columns.ListColumn(self._image.tags),
         ), )
-        self.assertEqual(datalist, tuple(data))
+        self.assertListItemEqual(datalist, tuple(data))
 
-    @mock.patch('openstackclient.api.utils.simple_filter')
+    @mock.patch('osc_lib.api.utils.simple_filter')
     def test_image_list_property_option(self, sf_mock):
         sf_mock.return_value = [copy.deepcopy(self._image)]
 
@@ -689,7 +727,7 @@ class TestImageList(TestImage):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.datalist, tuple(data))
+        self.assertListItemEqual(self.datalist, tuple(data))
 
     @mock.patch('osc_lib.utils.sort_items')
     def test_image_list_sort_option(self, si_mock):
@@ -708,27 +746,29 @@ class TestImageList(TestImage):
         )
         si_mock.assert_called_with(
             [self._image],
-            'name:asc'
+            'name:asc',
+            str,
         )
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.datalist, tuple(data))
+        self.assertListItemEqual(self.datalist, tuple(data))
 
     def test_image_list_limit_option(self):
+        ret_limit = 1
         arglist = [
-            '--limit', str(1),
+            '--limit', str(ret_limit),
         ]
         verifylist = [
-            ('limit', 1),
+            ('limit', ret_limit),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         columns, data = self.cmd.take_action(parsed_args)
         self.api_mock.image_list.assert_called_with(
-            limit=1, marker=self._image.id
+            limit=ret_limit, marker=None
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(len(self.datalist), len(tuple(data)))
+        self.assertEqual(ret_limit, len(tuple(data)))
 
     @mock.patch('osc_lib.utils.find_resource')
     def test_image_list_marker_option(self, fr_mock):
@@ -779,6 +819,66 @@ class TestImageList(TestImage):
             status='active', marker=self._image.id
         )
 
+    def test_image_list_tag_option(self):
+        arglist = [
+            '--tag', 'abc',
+        ]
+        verifylist = [
+            ('tag', 'abc'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        self.api_mock.image_list.assert_called_with(
+            tag='abc', marker=self._image.id
+        )
+
+
+class TestListImageProjects(TestImage):
+
+    project = identity_fakes.FakeProject.create_one_project()
+    _image = image_fakes.FakeImage.create_one_image()
+    member = image_fakes.FakeImage.create_one_image_member(
+        attrs={'image_id': _image.id,
+               'member_id': project.id}
+    )
+
+    columns = (
+        "Image ID",
+        "Member ID",
+        "Status"
+    )
+
+    datalist = ((
+        _image.id,
+        member.member_id,
+        member.status,
+    ))
+
+    def setUp(self):
+        super(TestListImageProjects, self).setUp()
+
+        self.images_mock.get.return_value = self._image
+        self.image_members_mock.list.return_value = self.datalist
+
+        self.cmd = image.ListImageProjects(self.app, None)
+
+    def test_image_member_list(self):
+        arglist = [
+            self._image.id
+        ]
+        verifylist = [
+            ('image', self._image.id)
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.image_members_mock.list.assert_called_with(self._image.id)
+
+        self.assertEqual(self.columns, columns)
+        self.assertEqual(len(self.datalist), len(tuple(data)))
+
 
 class TestRemoveProjectImage(TestImage):
 
@@ -1012,24 +1112,6 @@ class TestImageSet(TestImage):
             image_fakes.image_id, **kwargs)
         self.assertIsNone(result)
 
-    def test_image_set_with_unexist_owner(self):
-        self.project_mock.get.side_effect = exceptions.NotFound(None)
-        self.project_mock.find.side_effect = exceptions.NotFound(None)
-
-        arglist = [
-            '--owner', 'unexist_owner',
-            image_fakes.image_id,
-        ]
-        verifylist = [
-            ('owner', 'unexist_owner'),
-            ('image', image_fakes.image_id),
-        ]
-        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
-
-        self.assertRaises(
-            exceptions.CommandError,
-            self.cmd.take_action, parsed_args)
-
     def test_image_set_with_unexist_project(self):
         self.project_mock.get.side_effect = exceptions.NotFound(None)
         self.project_mock.find.side_effect = exceptions.NotFound(None)
@@ -1378,7 +1460,7 @@ class TestImageShow(TestImage):
         )
 
         self.assertEqual(image_fakes.IMAGE_columns, columns)
-        self.assertEqual(image_fakes.IMAGE_SHOW_data, data)
+        self.assertItemEqual(image_fakes.IMAGE_SHOW_data, data)
 
     def test_image_show_human_readable(self):
         self.images_mock.get.return_value = self.new_image
