@@ -12,9 +12,10 @@
 #   under the License.
 #
 
-import mock
-from mock import call
+from unittest import mock
+from unittest.mock import call
 
+from osc_lib.cli import format_columns
 from osc_lib import exceptions
 from osc_lib import utils
 
@@ -77,7 +78,7 @@ class TestTypeCreate(TestType):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_type_create_with_encryption(self):
         encryption_info = {
@@ -102,7 +103,7 @@ class TestTypeCreate(TestType):
         )
         encryption_data = (
             self.new_volume_type.description,
-            utils.format_dict(encryption_info),
+            format_columns.DictColumn(encryption_info),
             self.new_volume_type.id,
             True,
             self.new_volume_type.name,
@@ -138,7 +139,7 @@ class TestTypeCreate(TestType):
             body,
         )
         self.assertEqual(encryption_columns, columns)
-        self.assertEqual(encryption_data, data)
+        self.assertItemEqual(encryption_data, data)
 
 
 class TestTypeDelete(TestType):
@@ -246,7 +247,7 @@ class TestTypeList(TestType):
             t.id,
             t.name,
             t.is_public,
-            utils.format_dict(t.extra_specs),
+            format_columns.DictColumn(t.extra_specs),
         ))
 
     def setUp(self):
@@ -269,7 +270,7 @@ class TestTypeList(TestType):
         columns, data = self.cmd.take_action(parsed_args)
         self.types_mock.list.assert_called_once_with()
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, list(data))
+        self.assertListItemEqual(self.data, list(data))
 
     def test_type_list_with_options(self):
         arglist = [
@@ -283,7 +284,7 @@ class TestTypeList(TestType):
         columns, data = self.cmd.take_action(parsed_args)
         self.types_mock.list.assert_called_once_with()
         self.assertEqual(self.columns_long, columns)
-        self.assertEqual(self.data_long, list(data))
+        self.assertListItemEqual(self.data_long, list(data))
 
     def test_type_list_with_encryption(self):
         encryption_type = volume_fakes.FakeType.create_one_encryption_type(
@@ -302,13 +303,16 @@ class TestTypeList(TestType):
             self.volume_types[0].id,
             self.volume_types[0].name,
             self.volume_types[0].is_public,
-            utils.format_dict(encryption_info),
+            volume_type.EncryptionInfoColumn(
+                self.volume_types[0].id,
+                {self.volume_types[0].id: encryption_info}),
         ))
         encryption_data.append((
             self.volume_types[1].id,
             self.volume_types[1].name,
             self.volume_types[1].is_public,
-            '-',
+            volume_type.EncryptionInfoColumn(
+                self.volume_types[1].id, {}),
         ))
 
         self.encryption_types_mock.list.return_value = [encryption_type]
@@ -324,7 +328,7 @@ class TestTypeList(TestType):
         self.encryption_types_mock.list.assert_called_once_with()
         self.types_mock.list.assert_called_once_with()
         self.assertEqual(encryption_columns, columns)
-        self.assertEqual(encryption_data, list(data))
+        self.assertListItemEqual(encryption_data, list(data))
 
 
 class TestTypeSet(TestType):
@@ -443,7 +447,7 @@ class TestTypeShow(TestType):
             self.volume_type.id,
             True,
             self.volume_type.name,
-            utils.format_dict(self.volume_type.extra_specs)
+            format_columns.DictColumn(self.volume_type.extra_specs)
         )
 
         self.types_mock.get.return_value = self.volume_type
@@ -465,7 +469,7 @@ class TestTypeShow(TestType):
         self.types_mock.get.assert_called_with(self.volume_type.id)
 
         self.assertEqual(self.columns, columns)
-        self.assertEqual(self.data, data)
+        self.assertItemEqual(self.data, data)
 
     def test_type_show_with_encryption(self):
         encryption_type = volume_fakes.FakeType.create_one_encryption_type()
@@ -489,11 +493,11 @@ class TestTypeShow(TestType):
         )
         encryption_data = (
             self.volume_type.description,
-            utils.format_dict(encryption_info),
+            format_columns.DictColumn(encryption_info),
             self.volume_type.id,
             True,
             self.volume_type.name,
-            utils.format_dict(self.volume_type.extra_specs)
+            format_columns.DictColumn(self.volume_type.extra_specs)
         )
         arglist = [
             '--encryption-type',
@@ -509,7 +513,7 @@ class TestTypeShow(TestType):
         self.types_mock.get.assert_called_with(self.volume_type.id)
         self.encryption_types_mock.get.assert_called_with(self.volume_type.id)
         self.assertEqual(encryption_columns, columns)
-        self.assertEqual(encryption_data, data)
+        self.assertItemEqual(encryption_data, data)
 
 
 class TestTypeUnset(TestType):
@@ -587,3 +591,30 @@ class TestTypeUnset(TestType):
         result = self.cmd.take_action(parsed_args)
         self.encryption_types_mock.delete.assert_called_with(self.volume_type)
         self.assertIsNone(result)
+
+
+class TestColumns(TestType):
+
+    def test_encryption_info_column_with_info(self):
+        fake_volume_type = volume_fakes.FakeType.create_one_type()
+        type_id = fake_volume_type.id
+
+        encryption_info = {
+            'provider': 'LuksEncryptor',
+            'cipher': None,
+            'key_size': None,
+            'control_location': 'front-end',
+        }
+        col = volume_type.EncryptionInfoColumn(type_id,
+                                               {type_id: encryption_info})
+        self.assertEqual(utils.format_dict(encryption_info),
+                         col.human_readable())
+        self.assertEqual(encryption_info, col.machine_readable())
+
+    def test_encryption_info_column_without_info(self):
+        fake_volume_type = volume_fakes.FakeType.create_one_type()
+        type_id = fake_volume_type.id
+
+        col = volume_type.EncryptionInfoColumn(type_id, {})
+        self.assertEqual('-', col.human_readable())
+        self.assertIsNone(col.machine_readable())
